/*
 * This file is part of the Adblock Plus extension,
 * Copyright (C) 2006-2012 Eyeo GmbH
 *
 * Adblock Plus is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * Adblock Plus is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Adblock Plus.  If not, see <http://www.gnu.org/licenses/>.
 */

require("matcher").init();
with(require("filterClasses"))
{
  this.Filter = Filter;
  this.WhitelistFilter = WhitelistFilter;
}
with(require("subscriptionClasses"))
{
  this.Subscription = Subscription;
  this.DownloadableSubscription = DownloadableSubscription;
}
var FilterNotifier = require("filterNotifier").FilterNotifier;
var FilterStorage = require("filterStorage").FilterStorage;
var ElemHide = require("elemHide").ElemHide;
var defaultMatcher = require("matcher").defaultMatcher;
var Synchronizer = require("synchronizer").Synchronizer;

var isFirstRun = false;
FilterNotifier.addListener(function(action)
{
  if (action == "load")
  {
    importOldData();
    if (!window.localStorage.currentVersion)
    {
      isFirstRun = true;
      executeFirstRunActions();
    }
    window.localStorage.currentVersion = require("info").addonVersion;
  }
});

var toolbarButton;
var i18nMessages;

function importOldData()
{
  // TODO: Remove this once most people have the update
  if ("version" in widget.preferences || "last-version" in widget.preferences)
  {
    var oldLists = {
      "fanboy": "https://secure.fanboy.co.nz/fanboy-adblock.txt",
      "fanboy-ru": "https://secure.fanboy.co.nz/fanboy-russian.txt",
      "fanboy-es": "https://secure.fanboy.co.nz/fanboy-espanol.txt",
      "fanboy-ja": "https://secure.fanboy.co.nz/fanboy-japanese.txt",
      "fanboy-cz": "https://secure.fanboy.co.nz/fanboy-czech.txt",
      "fanboy-tr": "https://secure.fanboy.co.nz/fanboy-turkish.txt",
      "fanboy-pl": "https://secure.fanboy.co.nz/fanboy-polish.txt",
      "fanboy-vi": "https://secure.fanboy.co.nz/fanboy-vietnam.txt",
      "fanboy-sv": "https://secure.fanboy.co.nz/fanboy-swedish.txt",
      "fanboy-ko": "https://secure.fanboy.co.nz/fanboy-korean.txt",
      "fanboy-ann": "https://secure.fanboy.co.nz/fanboy-addon.txt",
      "easy": "https://easylist-downloads.adblockplus.org/easylist.txt",
      "easy-ru": "https://ruadlist.googlecode.com/svn/trunk/advblock.txt",
      "easy-de": "https://easylist-downloads.adblockplus.org/easylistgermany.txt",
      "easy-fr": "http://lian.info.tm/liste_fr.txt",
      "easy-zh": "https://adblock-chinalist.googlecode.com/svn/trunk/adblock.txt",
      "easy-id": "https://indonesianadblockrules.googlecode.com/hg/subscriptions/abpindo.txt",
      "easy-bg": "http://stanev.org/abp/adblock_bg.txt",
      "easy-ja": "https://adblock-plus-japanese-filter.googlecode.com/hg/abp_jp.txt",
      "easy-el": "https://www.void.gr/kargig/void-gr-filters.txt",
      "easy-pl": "http://adblocklist.org/adblock-pxf-polish.txt",
      "easy-nl": "https://dutchadblockfilters.googlecode.com/svn/trunk/AdBlock_Dutch_hide.txt",
      "easy-ro": "http://www.zoso.ro/pages/rolist.txt",
      "easy-fi": "http://www.wiltteri.net/wiltteri.txt",
      "easy-es": "http://abp.mozilla-hispano.org/nauscopio/filtros.txt",
      "easy-hu": "http://ajnasz.hu/adblock/recent",
      "adblock-lv": "https://gitorious.org/adblock-latvian/adblock-latvian/blobs/raw/master/lists/latvian-list.txt",
      "easy-priv": "https://easylist-downloads.adblockplus.org/easyprivacy.txt",
      "easy-ru-priv": "https://ruadlist.googlecode.com/svn/trunk/cntblock.txt"
    };

    // Import filter subscriptions
    for (var key in oldLists)
    {
      if (key in widget.preferences)
      {
        // Only add subscriptions that were enabled in old version
        if (widget.preferences[key] == "true")
        {
          var subscription = Subscription.fromURL(oldLists[key]);
          if (subscription && !(subscription.url in FilterStorage.knownSubscriptions))
          {
            FilterStorage.addSubscription(subscription);
            Synchronizer.execute(subscription);
          }
        }
      }
    }

    // Import custom filters
    if ("personal" in widget.preferences)
    {
      var customFilters = widget.preferences.personal.split("\n");
      for (var i = 0; i < customFilters.length; i++)
      {
        var filter = customFilters[i];
        if (filter === "")
          continue;

        if (filter.substr(0, 1) != "*")
          filter = "|" + filter;
        if (filter.substr(-1) != "*")
          filter = filter + "|";
        FilterStorage.addFilter(Filter.fromText(filter));
      }
    }

    // Import custom CSS rules
    if ("personal-css" in widget.preferences)
    {
      var customRules = widget.preferences["personal-css"].split("\n");
      for (var i = 0; i < customRules.length; i++)
      {
        var rule = customRules[i];
        if (rule === "")
          continue;

        rule = "##" + rule;
        FilterStorage.addFilter(Filter.fromText(rule));
      }
    }

    widget.preferences.clear();
  }
}

function executeFirstRunActions()
{
  // Don't do anything if the user has a subscription already
  var hasSubscriptions = FilterStorage.subscriptions.some(function(subscription) {return subscription instanceof DownloadableSubscription});
  if (hasSubscriptions)
    return;

  // Load subscriptions data
  var request = new XMLHttpRequest();
  request.open("GET", "subscriptions.xml");
  request.onload = function()
  {
    var subscriptions = request.responseXML.documentElement.getElementsByTagName("subscription");
    var selectedItem = null;
    var selectedPrefix = null;
    var matchCount = 0;
    for (var i = 0; i < subscriptions.length; i++)
    {
      var subscription = subscriptions[i];
      if (!selectedItem)
        selectedItem = subscription;

      var prefix = require("utils").Utils.checkLocalePrefixMatch(subscription.getAttribute("prefixes"));
      if (prefix)
      {
        if (!selectedPrefix || selectedPrefix.length < prefix.length)
        {
          selectedItem = subscription;
          selectedPrefix = prefix;
          matchCount = 1;
        }
        else if (selectedPrefix && selectedPrefix.length == prefix.length)
        {
          matchCount++;

          // If multiple items have a matching prefix of the same length:
          // Select one of the items randomly, probability should be the same
          // for all items. So we replace the previous match here with
          // probability 1/N (N being the number of matches).
          if (Math.random() * matchCount < 1)
          {
            selectedItem = subscription;
            selectedPrefix = prefix;
          }
        }
      }
    }

    var subscription = (selectedItem ? Subscription.fromURL(selectedItem.getAttribute("url")) : null);
    if (subscription)
    {
      subscription.disabled = false;
      subscription.title = selectedItem.getAttribute("title");
      subscription.homepage = selectedItem.getAttribute("homepage");
      if (subscription instanceof DownloadableSubscription && !subscription.lastDownload)
        Synchronizer.execute(subscription);
      FilterStorage.addSubscription(subscription);
    }
  };
  request.send(null);
}

function setDefaultOptions()
{
  function defaultOptionValue(opt, val)
  {
    if(!(opt in localStorage))
      localStorage[opt] = val;
  }

  defaultOptionValue("shouldShowIcon", "true");
}

function createToolbarButton()
{
  var properties = {
    disabled: false,
    title: "Adblock Plus",
    icon: "icons/abp-18.png"
  };

  toolbarButton = opera.contexts.toolbar.createItem(properties);
}

function refreshToolbarButton()
{
  var toolbar = opera.contexts.toolbar;
  if (localStorage["shouldShowIcon"] === "true")
    toolbar.addItem(toolbarButton);
  else
    toolbar.removeItem(toolbarButton);
}

function getJson(url)
{
  var request = new XMLHttpRequest();
  request.open("GET", url, false);
  request.responseType = "json";
  request.send();
  return request.response;
}

function loadMessagesForLocale(locale)
{
  var messagesFileName = "messages.json";
  var messagesPath;
  if (locale)
    messagesPath = "locales/" + locale + "/" + messagesFileName;
  else
    messagesPath = messagesFileName;

  var messages = getJson(messagesPath);
  if (!messages)
  {
    console.error("Failed to load messages for locale "
                  + (locale ? locale : "default"));
    return;
  }

  for (var i in messages)
    i18nMessages[i] = messages[i];
}

function loadI18nMessages()
{
  // Could be parsed from config.xml
  var defaultLocale = "en";

  i18nMessages = {};
  loadMessagesForLocale(defaultLocale);
  loadMessagesForLocale();
}

opera.extension.addEventListener("message", function(event)
{
  var request = event.data;
  switch (request.reqtype)
  {
    case "get-settings":
      var host = extractHostFromURL(request.frameUrl);
      var selectors = ElemHide.getSelectorsForDomain(host, false);
      event.source.postMessage({reqtype: "get-settings-response", selectors: selectors});
      break;
  }
}, false);

setDefaultOptions();
createToolbarButton();
refreshToolbarButton();
loadI18nMessages();
